<?php

namespace Drupal\Tests\k8s\Functional\Form;

use Drupal\Tests\k8s\Functional\K8sTestCase;

/**
 * Tests K8sCreateProjectForm.
 *
 * @group K8s
 */
class K8sCreateProjectFormTest extends K8sTestCase {

  public const K8S_PROJECT_REPEAT_COUNT = 3;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list k8s namespace',
      'view k8s namespace',
      'create k8s project',
    ];
  }

  /**
   * Tests creating a project.
   */
  public function testCreateProject() {

    $cloud_context = $this->cloudContext;

    // List Namespace for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/namespace");
    $this->assertResponse(200);

    // Create a new project.
    $add = $this->createProjectTestFormData(self::K8S_PROJECT_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_PROJECT_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $namespace = [];
      $namespace['name'] = $add[$i]['project_name'];
      $this->addNamespaceMockData($namespace);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/namespace/create_project",
        $add[$i],
        $this->t('Create')
      );
      $this->assertResponse(200);
      $this->assertText($this->t('The project "@name" has been created.', ['@name' => $add[$i]['project_name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/namespace");
      $this->assertResponse(200);
      $this->assertText($add[$i]['project_name']);
    }
  }

}
