<?php

namespace Drupal\cloud\Service\Util;

use Drupal\Core\Url;
use Drupal\Core\Utility\LinkGenerator;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Html generator utility for Entity link.
 */
class EntityLinkHtmlGenerator implements EntityLinkHtmlGeneratorInterface {

  /**
   * The link generator service.
   *
   * @var \Drupal\Core\Utility\LinkGenerator
   */
  protected $linkGenerator;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Utility\LinkGenerator $link_generator
   *   The link generator service.
   */
  public function __construct(LinkGenerator $link_generator) {
    $this->linkGenerator = $link_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('link_generator')
    );
  }

  /**
   * Render entity link for view.
   *
   * @param \Drupal\Core\Url $url
   *   The URL object.
   * @param int $id
   *   The entity ID.
   * @param string $name
   *   The field name of target entity.
   * @param string $alt_text
   *   Optional alternative text to display.
   *
   * @return string
   *   The HTML link string.
   */
  public function generate(Url $url, $id, $name = '', $alt_text = '') : string {
    $text = $id;
    if (!empty($alt_text)) {
      $text = $alt_text;
    }

    return $this->linkGenerator->generate($text, $url);
  }

}
